function [] = ctd2cdf(varargin)
% function [] = ctd2cdf(varargin);
%
% function to save CTD data into a one netcdf file per profile
%
% input  : fileName, the netcdf file
%          start, stop: indice to process
%
% example: 
% process all files
% ctd2cdf('amop-all_ctd.nc')
% process stations
% ctd2cdf('amop-all_ctd.nc', 00101, 01304)
%
% for debug and test only
% fileName = 'amop-all_ctd.nc';
% varargin = {'pirata-fr24-all_ctd.nc'};
% nargin = 1;

% check arguments
if nargin < 1
  nc = datagui.netcdf;
else
  nc = datagui.netcdf( varargin{1} );
end

% convert to OnceanSITES format
% ------------------------------
% convert time
year =  nc.Variables.REFERENCE_DATE_TIME.data__(1:4);
origin = datenum(str2double(year), 1, 1);

% convert dimensions
nc.Dimensions.DEPTH = nc.Dimensions.DEPH;
nc.Dimensions = rmfield(nc.Dimensions, 'DEPH');
%nc.Dimensions.TIME = nc.Dimensions.PROFILE;
nc.Dimensions = rmfield(nc.Dimensions, 'PROFILE');
nc.Dimensions = rmfield(nc.Dimensions, 'DATE_TIME');
s = struct('dimlen', 1, 'unlimited', 0);
nc.Dimensions.TIME      = s;
nc.Dimensions.LATITUDE  = s;
nc.Dimensions.LONGITUDE = s;
nc.Dimensions.POSITION  = s;

% convert variables
nc.Variables.LATITUDE = nc.Variables.LATX;
nc.Variables = rmfield(nc.Variables, 'LATX');
nc.Variables.LATITUDE.units = 'degrees_north';
nc.Variables.LATITUDE.long_name = 'Latitude of each location';
nc.Variables.LATITUDE.standard_name = 'latitude';
nc.Variables.LATITUDE.QC_indicator = 1;
nc.Variables.LATITUDE.QC_procedure = 5;


nc.Variables.LONGITUDE = nc.Variables.LONX;
nc.Variables = rmfield(nc.Variables, 'LONX');
nc.Variables.LONGITUDE.units = 'degrees_east';
nc.Variables.LATITUDE.long_name = 'Longitude of each location';
nc.Variables.LONGITUDE.standard_name = 'longitude';
nc.Variables.LONGITUDE.QC_indicator = 1;
nc.Variables.LONGITUDE.QC_procedure = 5;

nc.Variables.TIME = nc.Variables.DAYD;
julian = nc.Variables.TIME.data__ + origin;
nc.Variables.TIME.data__ = datenum(1950, 1, 1) + julian;
nc.Variables = rmfield(nc.Variables, 'DAYD');
nc.Variables = rmfield(nc.Variables, 'REFERENCE_DATE_TIME');
nc.Variables.TIME.units = 'days since 1950-01-01T00:00:00Z';
nc.Variables.TIME.long_name = 'time';
nc.Variables.TIME.standard_name = 'time';
nc.Variables.TIME.QC_indicator = 1;
nc.Variables.TIME.QC_procedure = 5;
if isfield(nc.Variables.TIME, 'JULIAN_DAY_ORIGIN')
  nc.Variables.TIME = rmfield(nc.Variables.TIME, 'JULIAN_DAY_ORIGIN');
end

nc.Variables.DEPTH = nc.Variables.DEPH;
nc.Variables = rmfield(nc.Variables, 'DEPH');
nc.Variables.DEPTH.units = 'meters';
nc.Variables.DEPTH.long_name = 'Depth of each measurement';
nc.Variables.DEPTH.standard_name = 'depth';
nc.Variables.DEPTH.positive = 'down';
nc.Variables.DEPTH.QC_indicator = 1;
nc.Variables.DEPTH.QC_procedure = 5;

for key =  keys(nc, 'Variables');
  if isfield(nc.Variables.(char(key)), 'name')
    nc.Variables.(char(key)) = rmfield(nc.Variables.(char(key)), 'name');
  end
end
  
% get profile list
profiles = nc.Variables.PROFILE.data__;

% get loop indices from station numbers
if nargin == 3
  start = find(profiles == varargin{2});
  stop  = find(profiles == varargin{3});
else  % process all
  start = 1;
  stop = length(profiles);
end

if isempty(start) || isempty(stop)
  error('bad values for start or stop');
end

% loop over each profiles
for i = start : stop
  
  % construct output filename from PROFILE variable
  fileNameOut = sprintf( 'data/nc/OS_FR24_201404_D_CTD%03d.nc', profiles(i));
  
  % open a new empty netcdf file
  ncout = datagui.netcdf( 'memory' );
  
  % write dimensions
  for key =  keys(nc, 'Dimensions');
    ncout.Dimensions.(char(key)) = nc.Dimensions.(char(key));
  end
  
  % write global attributes
  for k = keys(nc, 'Attributes')
    key = char(k);
    switch lower(key)
      case 'type'
        ncout.Attributes.data_type.data__ = 'OceanSITES profile data';
      case 'institut'
        ncout.Attributes.institution.data__ = nc.Attributes.(key).data__;
      case 'responsable'
        ncout.Attributes.contact.data__ = nc.Attributes.(key).data__;
      case 'date_creation'
        date = datestr(datenum(nc.Attributes.(key).data__,...
          'yyyymmddHHMMSS'), 'yyyy-mm-ddTHH:MM:SSZ');
        ncout.Attributes.date_update.data__ = date;
      case 'date_debut'
        date = datestr(datenum(nc.Attributes.(key).data__,...
          'yyyymmddHHMMSS'), 'yyyy-mm-ddTHH:MM:SSZ');
        ncout.Attributes.time_coverage_start.data__ = date;
      case 'date_fin'
        date = datestr(datenum(nc.Attributes.(key).data__,...
          'yyyymmddHHMMSS'), 'yyyy-mm-ddTHH:MM:SSZ');
        ncout.Attributes.time_coverage_end.data__ = date;
      otherwise
        ncout.Attributes.(lower(key)).data__ = nc.Attributes.(key).data__;
    end
  end
  ncout.Attributes.plateform_code.data__ = 'FZVM';
  ncout.Attributes.format_version.data__ = '1.2';
  ncout.Attributes.site_code.data__ = 'SISMER';
  ncout.Attributes.data_mode.data__ = 'D';
  ncout.Attributes.quality_control_indicator.data__ = 0;
  ncout.Attributes.history.data__ = ...
    strcat(datestr(now,'yyyy-mm-ddTHH:MM:SSZ'), ' data collected, J. Grelet');
  ncout.Attributes.Conventions.data__ = 'CF-1.4, OceanSITES 1.2';
  ncout.Attributes.Netcdf_version.data__ = '3.6';
  ncout.Attributes.author.data__ = 'Jacques Grelet';
  ncout.Attributes.contact.data__ = 'Jacques Grelet@ird.fr, Bernard.Bourles@ird.fr';
  ncout.Attributes.pi_name.data__ = 'Bernard Bourles IRD';
  ncout.Attributes.geospatial_lat_min.data__ = ...
    sprintf('%+4.2f', nc.Variables.LATITUDE.data__(i));
  ncout.Attributes.geospatial_lat_max.data__ = ...
     sprintf('%+4.2f', nc.Variables.LATITUDE.data__(i));
  ncout.Attributes.geospatial_lon_min.data__ = ...
     sprintf('%+5.2f', nc.Variables.LONGITUDE.data__(i));
  ncout.Attributes.geospatial_lon_max.data__ = ...
     sprintf('%+5.2f', nc.Variables.LONGITUDE.data__(i));

  % write variables
  % get all variables names and attributes in structures (name and att)
  for key =  keys(nc, 'Variables');
    att = fieldnames(nc.Variables.(char(key)));
    for iatt = 1 : length(att)
      ncout.Variables.(char(key)).(att{iatt}) = nc.Variables.(char(key)).(att{iatt});
      if strcmp(att{iatt}, 'data__')
        data = nc.Variables.(char(key)).data__;
        [~,n] = size(nc.Variables.(char(key)).data__);
        if n == 1
          ncout.Variables.(char(key)).data__ = data(i);
          %          elseif  strcmp(char(key), 'REFERENCE_DATE_TIME')
          %            ncout.Variables.(char(key)).data__ = data;
        else
          ncout.Variables.(char(key)).data__ = data(i,:);
        end
      end
    end
    % change dimensions
    switch length(ncout.Variables.(char(key)).dimension__)
      case 1
        ncout.Variables.(char(key)).dimension__ = {'TIME'};
      case 2
        ncout.Variables.(char(key)).dimension__ = {'TIME','DEPTH'};
      otherwise
        error('Matlab:ctd2cdf', ...
        'bad dimensions for %s\n%s.', char(key));
    end
      
  end
  
  % write each netcdf file
  ncout.write( fileNameOut, 'NC_CLOBBER' );
  
  % call destructot
  close( ncout );
  
end

% close file
close(nc);
